@@ -446,7 +446,7 @@ static void usage()
     V fprintf(stderr, "           -Vtimeout        Show hostnames that connect\n");
     V fprintf(stderr, "\n");
     V fprintf(stderr, "by John Walker\n");
-    V fprintf(stderr, "   WWW:    http://www.fourmilab.ch/\n");
+    V fprintf(stderr, "   http://www.fourmilab.ch/\n");
 }
 
 /*  Main program.  */
