@@ -906,7 +906,7 @@ static void usage()
     V fprintf(stderr, "           -Zpath     Write HTML private directory on base path\n");
     V fprintf(stderr, "\n");
     V fprintf(stderr, "by John Walker\n");
-    V fprintf(stderr, "   WWW:    http://www.fourmilab.ch/\n");
+    V fprintf(stderr, "   http://www.fourmilab.ch/\n");
 }
 
 /*  ADD_SDES_ITEM  --  Include an SDES item in a reply packet,
