@@ -26,7 +26,9 @@ struct destination {
     char ideakey[17];		      /* Destination IDEA key, if any */
     char pgpkey[17];		      /* Destination PGP key, if any */
     char blowfish_spec; 	      /* Nonzero if Blowfish key specified */
+#ifdef CRYPTO
     BF_KEY blowfishkey; 	      /* Destination Blowfish key, if any */
+#endif
     char *otp;			      /* Key file */
 };
 
@@ -35,7 +37,6 @@ static int sock;		      /* Communication socket */
 static struct destination *dests = NULL, *dtail;
 static int compressing = FALSE;       /* Compress sound: simple 2X */
 static int gsmcompress = TRUE;	      /* GSM compress buffers */
-static int gsm_byte_order_kludge = FALSE; /* Transition to correct byte order for GSM ? */
 #ifdef BYTE_SWAP_DEBUG
 static int gsm_byte_order_debug = FALSE; /* Send GSM length field with swapped bytes */
 #endif
@@ -62,6 +63,7 @@ static int agc = FALSE; 	      /* Automatic gain control active ? */
 static int rgain = 33;		      /* Current recording gain level */
 static int spurt = TRUE;	      /* True for first packet of talk spurt */
 static int debugging = FALSE;	      /* Debugging enabled here and there ? */
+static FILE *audioDumpFile = NULL;    /* Audio dump file, if any */
 static int havesound = FALSE;         /* True if we've acquired sound input */
 static char hostname[20];	      /* Host name to send with packets */
 static int loopback = FALSE;	      /* Remote loopback mode */
@@ -85,11 +87,11 @@ static char curvatkey[9] = "";        /* Current VAT DES key if currtpkey[0] !=
 static char curideakey[17] = "";      /* Current IDEA key if curideakey[0] != 0 */
 static char curpgpkey[17] = "";       /* Current PGP key if curpgpkey[0] != 0 */
 static char curblowfish_spec = FALSE; /* Nonzero if Blowfish key specified */
+#ifdef CRYPTO
 static BF_KEY curblowfishkey;	      /* Blowfish key */
-static char *curotp = NULL;	      /* Key file buffer */
-#ifdef TINY_PACKETS
-static int sound_packet;	      /* Ideal samples/packet */
 #endif
+static char *curotp = NULL;	      /* Key file buffer */
+static int sound_packet;	      /* Ideal samples/packet */
 static struct soundbuf *pgpsb = NULL; /* PGP key sound buffer, if any */
 static LONG pgpsbl;		      /* Length of PGP key sound buffer data */
 #ifdef HALF_DUPLEX
@@ -108,6 +110,12 @@ static long usticks;		      /* Microseconds per clock tick */
 extern int HPTermHookChar;
 #endif
 
+/* Audio input and control device file names. */
+
+#ifdef AUDIO_DEVICE_FILE
+extern char *devAudioInput, *devAudioControl;
+#endif
+
 #define ucase(x)    (islower(x) ? toupper(x) : (x))
 
 #ifndef USE_CURSES
@@ -215,56 +223,8 @@ static void gsmcomp(sb)
     }
 
     /* Hide original uncompressed buffer length in first 2 bytes of buffer. */
-
-    /* Allow me to explain the following if statement.	Speak
-       Freely for Unix prior to release 6.1e contained a bug
-       which caused it to both send (here) and receive
-       (in speaker.c) GSM packets with the embedded length
-       in host byte order.  This went undetected for a long
-       time because all the Unix machines I have access to
-       for testing are big-endian and thus interoperated
-       correctly with one another and the Windows version
-       (which does handle the byte order correctly).  Many
-       Linux and FreeBSD users only speak to one another,
-       and thus never noticed the problem.  It was only when
-       a little-endian Unix machine attempted to use GSM
-       compression in a connection with a Windows or
-       big-endian Unix machine that what came to be called
-       the "deafening static" problem manifested itself.
-       The fix is obvious, but we have to get there in
-       a cautious manner to avoid breaking interoperability
-       during the transition.  So here's the plan: starting
-       with Unix release 6.1e and the corresponding Windows
-       release, the GSM receiving code in both versions
-       will accept the GSM packet length field in
-       *either* byte order (a byte-swapped value in this
-       field can be detected unambiguously because if
-       byte-swapped it will exceed the actual length of
-       the packet, which is otherwise impossible).  Users
-       who install these releases will be able to receive
-       GSM sound from any version of Speak Freely on any
-       platform, present or future.  Version 6.1e for
-       Unix will, however, continue to send the GSM length
-       field in host byte order.  Why?	Because otherwise
-       Linux and FreeBSD users who communicate with one
-       another would both have to install 6.1e before
-       they could communicate once again.  To make the
-       transition smoother, 6.1e offers a "-tc" option,
-       which causes the GSM length field to be sent
-       in the correct byte order.  This option should be
-       set whenever a Unix user wishes to communicate
-       with a Windows user in GSM mode.  It will also
-       fix connections to big-endian Unix versions 
-       prior to 6.1e.  Once a majority of the installed
-       base is on the versions which don't care about
-       byte order in this field, sending in network byte
-       order will be the default.  */
-
-    if (gsm_byte_order_kludge) {
-	*((short *) sb->buffer.buffer_val) = htons((short) sb->buffer.buffer_len);
-    } else {
-	*((short *) sb->buffer.buffer_val) = sb->buffer.buffer_len;
-    }
+    
+    *((short *) sb->buffer.buffer_val) = htons((short) sb->buffer.buffer_len);
 #ifdef BYTE_SWAP_DEBUG
     if (gsm_byte_order_debug) {
 	unsigned char *bsp = (unsigned char *) sb->buffer.buffer_val;
@@ -361,6 +321,7 @@ static int sendrtpctrl(d, destaddr, msg, msgl, rtppkt)
 {
     char aux[1024];
 
+#ifdef CRYPTO
     if ((rtp || vat) && d->rtpdeskey[0]) {
 	int vlen;
 	des_cblock key;
@@ -402,6 +363,7 @@ static int sendrtpctrl(d, destaddr, msg, msgl, rtppkt)
 	    (des_cblock *) ivec, DES_ENCRYPT);
 	msg = aux;
     }
+#endif
     return sendto(sock, msg, msgl, 0, destaddr, sizeof d->ctrl);
 }
 
@@ -470,7 +432,9 @@ static int addest(host)
     bcopy(curideakey, d->ideakey, 17);
     bcopy(curpgpkey, d->pgpkey, 17);
     d->blowfish_spec = curblowfish_spec;
+#ifdef CRYPTO
     bcopy(&curblowfishkey, &(d->blowfishkey), sizeof(BF_KEY));
+#endif
     d->otp = curotp;
     d->time_to_live = ttl;
     d->name.sin_family = AF_INET;
@@ -512,6 +476,7 @@ static int addest(host)
 	sdes[0] = pid;
     }
 
+#ifdef CRYPTO
     if (pgpsb != NULL) {
 	int i;
 
@@ -530,6 +495,7 @@ static int addest(host)
 /* No need to sleep once ack from far end confirms key decoded. */
 sleep(7);
     }
+#endif
     return TRUE;
 }
 
@@ -631,6 +597,8 @@ static int sendpkt(sb)
 	    }
 	}
 #endif
+
+#ifdef CRYPTO
 	if (d->deskey[0] || d->ideakey[0] || d->blowfish_spec ||
 	    d->pgpkey[0] || d->otp != NULL) {
 	    soundbuf ebuf;
@@ -806,11 +774,13 @@ static int sendpkt(sb)
 	    for (i = 0; i < (lpc10compress ? robust : 1); i++) {
 		if (sendto(sock, (char *) &ebuf, pktlen,
 		    0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
-                    perror("sending datagram message");
+                    perror("sending encrypted audio");
 		    return FALSE;
 		}
 	    }
-	} else {
+	} else
+#endif
+	{
 	    int i, wasf = FALSE;
 
 	    if (!vat && !rtp) {
@@ -830,7 +800,7 @@ static int sendpkt(sb)
 	    for (i = 0; i < (lpc10compress ? robust : 1); i++) {
 		if (sendto(sock, (char *) sb, pktlen,
 		    0, (struct sockaddr *) &(d->name), sizeof d->name) < 0) {
-                    perror("sending datagram message");
+                    perror("sending audio");
 		    return FALSE;
 		}
 	    }
@@ -905,8 +875,8 @@ static int getaudio()
 #endif
 
     if (i <= 0) {
-            fprintf(stderr, "%s: unable to initialise audio.\n", progname);
-	    return FALSE;
+        fprintf(stderr, "%s: unable to initialise audio.\n", progname);
+	return FALSE;
     }
     havesound = TRUE;
     if (agc) {
@@ -964,6 +934,10 @@ static void exiting()
 
     freeaudio();
 
+    if (audioDumpFile != NULL) {
+	fclose(audioDumpFile);
+    }
+
 #ifdef PUSH_TO_TALK
     if (rawmode) {
         fprintf(stderr, "\r      \r");
@@ -1018,6 +992,19 @@ static void exiting()
     exit(0);
 }
 
+/*  CHANGEMODE	--  Reset state when talk/pause/quiet mode changes.  */
+
+static void changemode()
+{
+    if (talking) {
+	spurt = TRUE;
+	/* Discard all backlog sound input. */
+	soundflush();
+    } else {
+	sqwait = 0;		      /* Reset squelch delay */
+    }
+}
+
 /*  TERMCHAR  --  Check for special characters from console when
 		  in raw mode.	*/
 
@@ -1088,6 +1075,7 @@ fprintf(stderr, "\nOut of chat mode, input(%d) = \"%s\"\n", strlen(s), s);
 		    tty_rawmode();
 #endif
 		}
+		changemode();
 		break;
 	    }
 	}
@@ -1118,8 +1106,6 @@ static int sendfile(f)
 	ularm(TimerStep);
     }
 
-#ifdef TINY_PACKETS
-
     /* Compute the number of sound samples needed to fill a
        packet of TINY_PACKETS bytes. */
 
@@ -1146,7 +1132,6 @@ static int sendfile(f)
 
 #ifdef SHOW_PACKET_SIZE
     printf("Samples per packet = %d\n", sound_packet);
-#endif
 #endif
     lread = sound_packet;
 
@@ -1204,11 +1189,7 @@ static int sendfile(f)
 
 	    (bread = nread =
 		fread(buf + (toasted ? 2 : 0), 1,
-#ifdef TINY_PACKETS
 				  lread,
-#else
-				  (gsmcompress ? 1600 : BUFL),
-#endif
 				  afile)) > 0) {
 	    netbuf.compression = fProtocol | (ring ? (fSetDest | fDestSpkr) : 0);
 	    ring = FALSE;
@@ -1220,19 +1201,10 @@ static int sendfile(f)
 		netbuf.compression |= fCompGSM;
 	    } else {
 		if (compressing) {
-#ifdef OLD2X
-		    int i;
-
-		    nread /= 2;
-		    for (i = 1; i < nread; i++) {
-			buf[i] = buf[i * 2];
-		    }
-#else
 		    int is = nread, os = nread / 2;
 
 		    rate_flow(buf, buf, &is, &os);
 		    nread = os;
-#endif
 		    netbuf.compression |= fComp2X;
 		}
 		netbuf.buffer.buffer_len = nread;
@@ -1271,11 +1243,7 @@ static int sendfile(f)
 	       to neither convention, we just guess based on the number
 	       of samples in the packet and a fudge factor. */
 
-#ifdef TINY_PACKETS
 #define kOverhead   8000
-#else
-#define kOverhead   3000
-#endif
 
 #ifdef BSD_like
 	    ftime(&t2);
@@ -1392,31 +1360,34 @@ printf("Packet time %d Delta t = %d Error = %d, Corr = %d, Wait = %d\n",
 	/* Send real-time sound. */
 
 #ifdef Solaris
-	setaubufsize(
-#ifdef TINY_PACKETS
-	    sound_packet
-#else
-	    sizeof buf
-#endif
-		    );
+	setaubufsize(sound_packet);
 #endif
 
 	if (!getaudio()) {
             fprintf(stderr, "Unable to initialise audio.\n");
 	    return 2;
 	}
+	changemode();
 	while (TRUE) {
 	    int soundel = 0;
 	    unsigned char *bs = (unsigned char *) buf;
 
 	    if (havesound) {
-		soundel = soundgrab(buf,
-#ifdef TINY_PACKETS
-					 sound_packet
-#else
-					 sizeof buf
-#endif
-				   );
+
+                /* Obtain a packet's worth of real-time audio from
+		   the audio input hardware. */
+
+		soundel = soundgrab(buf, sound_packet);
+
+		/* If a raw audio dump file (-w option) is specified,
+		   dump whatever we got from the audio input to the
+		   designated file.  This can come in handy when
+                   debugging audio drivers which don't provide samples
+		   in the expected 8 bit mu-law mode. */
+
+		if ((audioDumpFile != NULL) && (soundel > 0)) {
+		    fwrite(buf, 1, soundel, audioDumpFile);
+		}
 	    }
 
 #ifdef PUSH_TO_TALK
@@ -1456,13 +1427,7 @@ printf("Packet time %d Delta t = %d Error = %d, Corr = %d, Wait = %d\n",
 		    fprintf(stderr, talking ? (squelch > 0 ?
                             "\rQuiet: " : "\rTalk:   ") : "\rPause: ");
 		    fcntl(fileno(stdin), F_SETFL, talking ? O_NDELAY : 0);
-		    if (talking) {
-			spurt = TRUE;
-			/* Discard all backlog sound input. */
-			soundflush();
-		    } else {
-			sqwait = 0;   /* Reset squelch delay */
-		    }
+		    changemode();
 		    break;
 		} else {
 		    if (rlen == -1 && errno == EINTR) {
@@ -1557,19 +1522,10 @@ printf("Packet time %d Delta t = %d Error = %d, Corr = %d, Wait = %d\n",
 
 		    ring = FALSE;
 		    if (compressing) {
-#ifdef OLD2X
-			int i;
-
-			soundel /= 2;
-			for (i = 1; i < soundel; i++) {
-			    buf[i] = buf[i * 2];
-			}
-#else
 			int is = soundel, os = soundel / 2;
 
 			rate_flow(buf, buf, &is, &os);
 			soundel = os;
-#endif
 			netbuf.compression |= fComp2X;
 		    }
 		    netbuf.buffer.buffer_len = soundel;
@@ -1637,10 +1593,15 @@ static void makeSessionKey(key, seed, mode)
     MD5Init(&md5c);
     MD5Update(&md5c, s, strlen(s));
     MD5Final(md5key1, &md5c);
+#ifdef CRYPTO
     init_idearand(md5key, md5key1, time(NULL));
+#define nextrand    idearand()
+#else
+#define nextrand    (md5key[j] ^ md5key1[j])
+#endif
     if (mode) {
 	for (j = k = 0; j < 16; j++) {
-	    unsigned char rb = idearand();
+	    unsigned char rb = nextrand;
 
 #define Rad16(x) ((x) + 'A')
 	    key[k++] = Rad16((rb >> 4) & 0xF);
@@ -1653,10 +1614,13 @@ static void makeSessionKey(key, seed, mode)
 	key[--k] = 0;
     } else {
 	for (j = 0; j < 16; j++) {
-	    key[j] = idearand();
+	    key[j] = nextrand;
 	}
     }
+#ifdef CRYPTO
     close_idearand();
+#endif
+#undef nextrand
 }
 
 /*  PROG_NAME  --  Extract program name from argv[0].  */
@@ -1682,36 +1646,60 @@ static void usage()
     V fprintf(stderr, "           -A         Always transmit unless squelched\n");
     V fprintf(stderr, "     *     -B         Push to talk using keyboard\n");
 #endif
+#ifdef CRYPTO
     V fprintf(stderr, "           -BFkey     Blowfish encrypt with key\n");
+#endif
     V fprintf(stderr, "           -C         Compress subsequent sound\n");
     V fprintf(stderr, "           -D         Enable debug output\n");
+#ifdef CRYPTO
     V fprintf(stderr, "           -E[key]    Emit session key string\n");
+#endif
     V fprintf(stderr, "           -F         ADPCM compression\n");
     V fprintf(stderr, "           -G         Automatic gain control\n");
+#ifdef CRYPTO
     V fprintf(stderr, "           -Ikey      IDEA encrypt with key\n");
     V fprintf(stderr, "           -Kkey      DES encrypt with key\n");
+#endif
     V fprintf(stderr, "           -L         Remote loopback\n");
     V fprintf(stderr, "           -LPC       LPC compression\n");
     V fprintf(stderr, "           -LPC10Rn   LPC-10 compression, n copies\n");
     V fprintf(stderr, "     *     -M         Manual record gain control\n");
     V fprintf(stderr, "           -N         Do not compress subsequent sound\n");
+#ifdef CRYPTO
     V fprintf(stderr, "           -Ofile     Use file as key file\n");
+#endif
     V fprintf(stderr, "           -Phostname[:port] Party line, add host to list\n");
     V fprintf(stderr, "     *     -Q         Disable debug output\n");
     V fprintf(stderr, "           -R         Ring--force volume, output to speaker\n");
     V fprintf(stderr, "           -RTP       Use Internet Real-Time Protocol\n");
     V fprintf(stderr, "           -Sn,t      Squelch at level n (0-32767), timeout t milliseconds\n");
+ 
     V fprintf(stderr, "     *     -T         Telephone (GSM) compression\n");
-    V fprintf(stderr, "           -TC        GSM with corrected x86 byte order\n");
 #ifdef BYTE_SWAP_DEBUG
     V fprintf(stderr, "           -TD        Debug: send GSM with swapped byte order\n");
 #endif
     V fprintf(stderr, "           -U         Print this message\n");
     V fprintf(stderr, "           -VAT       Use VAT protocol\n");
+    V fprintf(stderr, "           -Wfile     Write raw audio input to file\n");
+#ifdef AUDIO_DEVICE_FILE
+    V fprintf(stderr, "           -Yindev[:ctldev] Override default audio device file name or specify open #fd\n");
+#endif
+#ifdef CRYPTO
     V fprintf(stderr, "           -Z\"user..\" Send PGP session key for user(s)\n");
+#endif
     V fprintf(stderr, "\n");
     V fprintf(stderr, "by John Walker\n");
-    V fprintf(stderr, "   WWW:    http://www.fourmilab.ch/\n");
+    V fprintf(stderr, "   http://www.fourmilab.ch/\n");
+#ifndef CRYPTO
+    V fprintf(stderr, "\n");
+    V fprintf(stderr, "Note: This version of Speak Freely was built with\n");
+    V fprintf(stderr, "      encryption code removed to permit redistribution\n");
+    V fprintf(stderr, "      without concern for export control and regulations\n");
+    V fprintf(stderr, "      concerning encryption technology in some jurisdictions.\n");
+    V fprintf(stderr, "      You can download a version of Speak Freely with\n");
+    V fprintf(stderr, "      full encryption from:\n");
+    V fprintf(stderr, "             http://www.fourmilab.ch/speakfree/unix/\n");
+#endif
 }
 
 /*  Main program.  */
@@ -1728,10 +1716,14 @@ main(argc, argv)
     char s[1024];
 
     progname = prog_name(argv[0]);
-    gethostname(hostname, sizeof hostname);
-    if (strlen(hostname) > 15) {
-	hostname[15] = 0;
+    if (gethostname(hostname, sizeof hostname) == -1) {
+        strcpy(hostname, "localhost");
+    } else {
+	if (strlen(hostname) > 15) {
+	    hostname[15] = 0;
+	}
     }
+
 #ifdef LINUX_FPU_FIX
     __setfpucw(_FPU_IEEE);	      /* Mask floating point interrupts to
 					 enable standard IEEE processing.
@@ -1739,9 +1731,7 @@ main(argc, argv)
 					 default, so this is needed only on
 					 older libraries. */
 #endif
-#ifndef OLD2X
     rate_start(8000, 4000);
-#endif
     gsmh = gsm_create();
 
     /* Initialise LPC decoding. */
@@ -1759,15 +1749,15 @@ main(argc, argv)
 
 #ifdef HALF_DUPLEX
     {
-	struct hostent *h;
-	char host[512];
-
-	gethostname(host, sizeof host);
-	h = gethostbyname(host);
-	bcopy((char *) (h->h_addr), (char *) &localhost,
-	    sizeof localhost);
+#ifndef INADDR_LOOPBACK
+	/* The standard local host loopback address is supposed to be
+	   defined in <netinet/in.h>, but just in case some screwball
+           system doesn't, let's include our own definition here. */
+#define INADDR_LOOPBACK  0x7F000001
+#endif
+	localhost.s_addr = htonl(INADDR_LOOPBACK);
 #ifdef HDX_DEBUG
-        fprintf(stderr, "%s: local host %s = %s\n", progname, host,
+        fprintf(stderr, "%s: local host %s\n", progname,
 		inet_ntoa(localhost));
 #endif
     }
@@ -1830,6 +1820,7 @@ main(argc, argv)
 		    break;
 
                 case 'B':             /* -B  -- Push to talk (button) */
+#ifdef CRYPTO
 				      /* -BFkey  -- Set Blowfish key */ 
                     if (ucase(op[1]) == 'F') {
 			if (strlen(op + 2) == 0) {
@@ -1851,10 +1842,13 @@ main(argc, argv)
 			    }
 			}
 		    } else {
+#endif
 			if (isatty(fileno(stdin))) {
 			    push = TRUE;
 			}
+#ifdef CRYPTO
 		    }
+#endif
 		    break;
 #endif
 
@@ -1866,10 +1860,12 @@ main(argc, argv)
 		    debugging = TRUE;
 		    break;
 
+#ifdef CRYPTO
                 case 'E':             /* -E  --  Emit session key and exit */
 		    makeSessionKey(s, op + 1, TRUE);
                     printf("%s\n", s);
 		    return 0;
+#endif
 
                 case 'F':             /* -F -- ADPCM compression */
 		    adpcmcompress = TRUE;
@@ -1880,6 +1876,7 @@ main(argc, argv)
 		    agc = TRUE;
 		    break;
 
+#ifdef CRYPTO
                 case 'I':             /* -Ikey  --  Set IDEA key */
 		    if (strlen(op + 1) == 0) {
 			curideakey[0] = FALSE;
@@ -1944,6 +1941,7 @@ main(argc, argv)
 			}
 		    }
 		    break;
+#endif
 
                 case 'L':             /* -L      --  Remote loopback */
 				      /* -LPC	 --  LPC compress sound */
@@ -1985,6 +1983,7 @@ main(argc, argv)
 		    adpcmcompress = FALSE;
 		    break;
 
+#ifdef CRYPTO
                 case 'O':             /* -Ofile -- Use file as key file */
 		    if (op[1] == 0) {
 			curotp = NULL; /* Switch off key file */
@@ -2020,6 +2019,7 @@ main(argc, argv)
 			}
 		    }
 		    break;
+#endif
 
                 case 'P':             /* -Phost  --  Copy output to host  */
 		    if (!addest(op + 1)) {
@@ -2060,13 +2060,16 @@ main(argc, argv)
 		    break;
 
                 case 'T':             /* -T -- Telephone (GSM) compression */
-				      /* -TC -- GSM transition with swapped length byte order */
 		    gsmcompress = TRUE;
 		    lpccompress = lpc10compress = adpcmcompress = FALSE;
-                    if (ucase(op[1]) == 'C') {
-			gsm_byte_order_kludge = TRUE;
-		    }
 #ifdef BYTE_SWAP_DEBUG
+		    /* This can be used if you absolutely have to communicate
+		       with somebody running a version prior to 6.1e on little-endian
+		       hardware.  This brakes the byte order of GSM packets to
+		       compensate for a bug in earlier versions.  Note that you
+		       should only set this for compatibility with old Unix
+		       versions--the problem this works around was never in
+		       the Windows version. */
                     if (ucase(op[1]) == 'D') {
 			gsm_byte_order_debug = TRUE;
 		    }
@@ -2085,6 +2088,25 @@ main(argc, argv)
 		    }
 		    break;
 
+                case 'W':             /* -Wfile -- Write raw audio to file */
+                    audioDumpFile = fopen(op + 1, "w");
+		    if (audioDumpFile == NULL) {
+                        perror("Cannot create raw audio dump (-w option) file");
+			return 2;
+		    }
+		    break;
+
+#ifdef AUDIO_DEVICE_FILE
+                case 'Y':             /* -Yindev:[ctldev] -- Specify audio input and control device file names
+							     or #open_fd. */
+		    devAudioInput = op + 1;
+                    if (strchr(op + 1, ':') != NULL) {
+                        devAudioControl = strchr(op + 1, ':') + 1;
+		    }
+		    break;
+#endif
+
+#ifdef CRYPTO
                 case 'Z':             /* -Z"user1 user2..."  --  Send PGP
 						 encrypted session key to
 						 named users */
@@ -2141,12 +2163,10 @@ main(argc, argv)
 #ifdef PGP_DEBUG
                             fprintf(stderr, "PGP key buffer length: %d\n", flen);
 #endif
-#ifdef TINY_PACKETS
 			    if (flen > (TINY_PACKETS -
 				    (sizeof(soundbuf) - BUFL))) {
                                 fprintf(stderr, "Warning: PGP key message exceeds %d packet size.\n", TINY_PACKETS);
 			    }
-#endif
 			    if (pgpsb != NULL) {
 				free(pgpsb);
 			    }
@@ -2175,6 +2195,7 @@ main(argc, argv)
 			unlink(f);
 		    }
 		    break;
+#endif
 	    }
 	} else {
 
@@ -2186,16 +2207,12 @@ main(argc, argv)
 	    }
 
 	    if (rtp || vat) {
+#ifdef CRYPTO
 		if (curideakey[0] || curpgpkey[0] ||
 		    curblowfish_spec || curotp != NULL) {
                     fprintf(stderr, "Blowfish, IDEA, PGP, and Key File encryption cannot be used\n    with -RTP or -VAT protocols.\n");
 		    return 2;
 		}
-#ifdef ZZZZ
-		if (vat && curkey[0]) {
-                    fprintf(stderr, "DES encryption not implemented for VAT protocol.\n");
-		    return 2;
-		}
 #endif
 		if (compressing) {
                     fprintf(stderr, "Simple (-C) compression cannot be used with\n   -RTP or -VAT protocols.\n");
@@ -2234,6 +2251,8 @@ main(argc, argv)
     exiting();
     gsm_destroy(gsmh);
     lpc_end();
+#ifdef CRYPTO
     desdone();
+#endif
     return 0;
 }
