@@ -666,7 +666,7 @@ static void usage()
     V fprintf(stderr, "           -Vtimeout        Show hostnames that connect\n");
     V fprintf(stderr, "\n");
     V fprintf(stderr, "by John Walker\n");
-    V fprintf(stderr, "   WWW:    http://www.fourmilab.ch/\n");
+    V fprintf(stderr, "   http://www.fourmilab.ch/\n");
 }
 
 /*  Main program.  */
@@ -992,6 +992,7 @@ main(argc, argv)
 	    if (c != NULL) {
 		struct hostent *h;
 
+		memset(c, 0, sizeof(struct connection));  /***** Added by Rod *****/
 		newconn = TRUE;
 		c->con_next = conn;
 		conn = c;
@@ -1037,6 +1038,7 @@ main(argc, argv)
                     fprintf(stderr, "%s: %s %s connect\n", prog, etime(FALSE), c->con_hostname);
 		}
 		c->con_bye = FALSE;
+		changed();               /******* Added by Rod *******/
 	    }
 	} else {
 	    crit = FALSE;
