@@ -24,7 +24,9 @@ static char rtpdeskey[9] = "";        /* RTP DES key, if any */
 static char vatdeskey[9] = "";        /* VAT DES key, if any */
 static char ideakey[17] = "";         /* IDEA key, if any */
 static char blowfish_spec = FALSE;    /* Nonzero if Blowfish key specified */
+#ifdef CRYPTO
 static BF_KEY blowfishkey;	      /* Blowfish key */
+#endif
 static char *curotp = NULL;	      /* Key file, if any */
 static int whichport = Internet_Port; /* Port to listen on (base address) */
 static int sock;		      /* Data socket */
@@ -49,6 +51,7 @@ static char *prog;		      /* Program name */
 static FILE *record = NULL;	      /* Answering machine record file */
 static char *pgppass = NULL;	      /* PGP secret key password */
 static char *replyfile = NULL;	      /* Reply file pathname */
+static char *replycmdexe = NULL;
 static char *replycmd = "sfmike -t";  /* Default reply command */
 static char *busysignal = "sleep 10; sfmike %s busy.au"; /* Default busy signal command */
 static int busytimeout = 60 * 1000000L; /* Busy signal timeout interval */
@@ -82,6 +85,12 @@ static int facesDisplayed = 0;	      /* Face viewers currently active */
 
 static struct connection *conn = NULL;/* Chain of current connections */
 
+/* Audio output and control device file names. */
+
+#ifdef AUDIO_DEVICE_FILE
+extern char *devAudioOutput, *devAudioControl;
+#endif
+
 #define Debug	((debugforce != 0) ? (debugforce > 0) : debugging)
 
 #define TickTock    (10 * 1000000L)   /* Alarm interval in microseconds */
@@ -252,11 +261,19 @@ static void makeSessionKey(key)
     MD5Init(&md5c);
     MD5Update(&md5c, s, strlen(s));
     MD5Final(md5key1, &md5c);
+#ifdef CRYPTO
     init_idearand(md5key, md5key1, time(NULL));
+#endif
     for (j = 0; j < 16; j++) {
+#ifdef CRYPTO
 	key[j] = idearand();
+#else
+	key[j] = md5key[j] ^ md5key1[j];
+#endif
     }
+#ifdef CRYPTO
     close_idearand();
+#endif
 }
 
 /*  ULARM  --  Wrapper for setitimer() that looks like alarm()
@@ -692,9 +709,9 @@ static void viewerterm()
 		  wait3(&status, WNOHANG, 0)
 #endif
 	   ) > 0) {
-if (Debug) {
-  fprintf(stderr, "Child process %d done\n", pid);
-}
+	if (Debug) {
+            fprintf(stderr, "Child process %d done\n", pid);
+	}
 	if (WIFEXITED(status) || WIFSIGNALED(status)) {
 	    struct connection *c = conn;
 
@@ -763,6 +780,8 @@ static void playbuffer(msg, c)
 	}
     }
 
+#ifdef CRYPTO
+
     /* If message is encrypted, decrypt. */
 
     if ((msg->compression & fEncOTP) && (curotp != NULL)) {
@@ -842,6 +861,21 @@ static void playbuffer(msg, c)
 	    bcopy(tbuf, bbuf, 8);
 	}
     }
+#else
+    if ((msg->compression & (fEncDES | fEncOTP | fEncIDEA | fEncPGP | fEncBF)) &&
+	!(c->con_crypt_warning)) {
+	c->con_crypt_warning = TRUE;
+        fprintf(stderr, "%s: Warning: host %s is sending encrypted audio\n",
+		prog, c->con_hostname);
+        fprintf(stderr, "   which cannot be decoded by this version of\n");
+        fprintf(stderr, "   Speak Freely, which was built with encryption\n");
+        fprintf(stderr, "   removed to permit redistribution without concern\n");
+        fprintf(stderr, "   for export control and other regulations.\n");
+        fprintf(stderr, "   You can download a version of Speak Freely which\n");
+        fprintf(stderr, "   includes full encryption from:\n");
+        fprintf(stderr, "       http://www.fourmilab.ch/speakfree/unix/\n");
+    }
+#endif
 
     /* If message is compressed, decompress appropriately. */
 
@@ -871,28 +905,10 @@ static void playbuffer(msg, c)
     }
 
     if (msg->compression & fComp2X) {
-#ifdef OLD2X
-	int i;
-	register char *ab = auxbuf;
-
-	assert(len < sizeof auxbuf);
-	for (i = 0; i < len; i++) {
-	    *ab++ = i == 0 ? *val :
-#define SMOOTH_2X
-#ifdef SMOOTH_2X
-		(audio_s2u((audio_u2s(*val) + audio_u2s(val[-1])) / 2));   
-#else
-		val[-1];
-#endif
-	    *ab++ = *val++;
-	}
-	len *= 2;
-#else
 	int is = len, os = len * 2;
 
 	rate_flow(val, auxbuf, &is, &os);
 	len = os;
-#endif
 	val = auxbuf;
     }
 
@@ -966,17 +982,26 @@ static void usage()
     V fprintf(stderr, "Usage: %s [options]\n", prog);
     V fprintf(stderr, "Options:\n");
     V fprintf(stderr, "           -A\"file command\" Create answer command in file\n");
-    V fprintf(stderr, "           -B\"command\"    Execute command for busy signal\n");
+    V fprintf(stderr, "           -B\"command\"      Execute command for busy signal\n");
+#ifdef CRYPTO
     V fprintf(stderr, "           -BFkey           Blowfish decrypt with key\n");
+#endif
     V fprintf(stderr, "           -D               Force debug output\n");
+    V fprintf(stderr, "           -E\"command\"      Execute command on incoming call\n");
+#ifdef CRYPTO
     V fprintf(stderr, "           -Ikey            IDEA decrypt with key\n");
+#endif
     V fprintf(stderr, "           -Jwait,idle      Jitter delay wait and idle in milliseconds\n");
+#ifdef CRYPTO
     V fprintf(stderr, "           -Kkey            DES decrypt with key\n");
+#endif
 #ifdef MULTICAST
     V fprintf(stderr, "           -Mhost/ip        Join multicast to given name or IP address\n");
 #endif
     V fprintf(stderr, "           -N               Disable remote ring requests\n");
+#ifdef CRYPTO
     V fprintf(stderr, "           -Ofile           Use file as key file\n");
+#endif
     V fprintf(stderr, "           -Pport           Listen on given port\n");
     V fprintf(stderr, "           -Q               Prevent debug output\n");
     V fprintf(stderr, "           -R[+]file        Record [append] sound in file\n");
@@ -986,10 +1011,25 @@ static void usage()
 #ifdef HEXDUMP
     V fprintf(stderr, "           -X               Dump packets in hex\n");
 #endif
+#ifdef AUDIO_DEVICE_FILE
+    V fprintf(stderr, "           -Youtdev[:ctldev] Override default audio device file name or specify open #fd\n");
+#endif
+#ifdef CRYPTO
     V fprintf(stderr, "           -Z\"phrase\"       Set PGP secret key pass phrase\n");
+#endif
     V fprintf(stderr, "\n");
     V fprintf(stderr, "by John Walker\n");
-    V fprintf(stderr, "   WWW:    http://www.fourmilab.ch/\n");
+    V fprintf(stderr, "   http://www.fourmilab.ch/\n");
+#ifndef CRYPTO
+    V fprintf(stderr, "\n");
+    V fprintf(stderr, "Note: This version of Speak Freely was built with\n");
+    V fprintf(stderr, "      encryption code removed to permit redistribution\n");
+    V fprintf(stderr, "      without concern for export control and regulations\n");
+    V fprintf(stderr, "      concerning encryption technology in some jurisdictions.\n");
+    V fprintf(stderr, "      You can download a version of Speak Freely with\n");
+    V fprintf(stderr, "      full encryption from:\n");
+    V fprintf(stderr, "             http://www.fourmilab.ch/speakfree/unix/\n");
+#endif
 }
 
 /*  Main program.  */
@@ -1014,7 +1054,7 @@ main(argc, argv)
 
     /* First pass option processing.  We have to first scan
        the options to handle any which affect creation of the
-       socket.	One the second pass we can assume the socket
+       socket.	On the second pass we can assume the socket
        already exists, allowing us to join multicast groups,
        etc. */
 
@@ -1124,9 +1164,14 @@ main(argc, argv)
 			    }
 			    replyfile = op + 1;
 			}
+			break;		  
+
+                    case 'E':             /* -E"command"  --  Execute command on connect */
+			replycmdexe = op + 1;
 			break;
 
                     case 'B':             /* -B"command"  --  Execute command for busy signal. */
+#ifdef CRYPTO
 					  /* -BFkey  -- Set Blowfish key */ 
                         if (ucase(op[1]) == 'F') {
 			    if (strlen(op + 2) == 0) {
@@ -1148,7 +1193,9 @@ main(argc, argv)
                                     fprintf(stderr, "\n");
 				}
 			    }
-			} else {
+			} else
+#endif
+			{
 			    if (strlen(op + 1) > 0) {
                                 if (!strstr(op + 1, "%s")) {
 				    fprintf(stderr,
@@ -1164,6 +1211,7 @@ main(argc, argv)
 			}
 			break;
 
+#ifdef CRYPTO
                     case 'I':             /* -Ikey  --  Set IDEA key */
 			if (strlen(op + 1) == 0) {
 			    ideakey[0] = FALSE;
@@ -1183,6 +1231,7 @@ main(argc, argv)
 			    }
 			}
 			break;
+#endif
 
                     case 'J':             /* -Jwait,idle --  Set jitter wait and idle times */
 			if (isdigit(*(op + 1))) {
@@ -1211,6 +1260,7 @@ main(argc, argv)
 			}
 			break;
 
+#ifdef CRYPTO
                     case 'K':             /* -Kkey  --  Set DES key */
 			desinit(1);	  /* Initialise the DES library */
 			if (strlen(op + 1) == 0) {
@@ -1254,6 +1304,7 @@ main(argc, argv)
 			    }
 			}
 			break;
+#endif
 
 #ifdef MULTICAST
                     case 'M':             /* -Mhost/ip -- Join multicast to name/IP number */
@@ -1288,6 +1339,7 @@ main(argc, argv)
 			noring = TRUE;
 			break;
 
+#ifdef CRYPTO
                     case 'O':             /* -Ofile -- Use file as key file */
                         fp = fopen(op + 1, "r");
 			if (fp == NULL) {
@@ -1319,6 +1371,7 @@ main(argc, argv)
 			    }
 			}
 			break;
+#endif
 
                     case 'Q':             /* -Q  --  Prevent debug output */
 			debugforce = -1;
@@ -1366,6 +1419,18 @@ main(argc, argv)
 			break;
 #endif
 
+#ifdef AUDIO_DEVICE_FILE
+                case 'Y':             /* -Youtdev:[ctldev] -- Specify audio
+						     output and control device
+						     file names or #open_fd. */
+		    devAudioOutput = op + 1;
+                    if (strchr(op + 1, ':') != NULL) {
+                        devAudioControl = strchr(op + 1, ':') + 1;
+		    }
+		    break;
+#endif
+
+#ifdef CRYPTO
                     case 'Z':             /* -Z"phrase"  --  Set PGP pass phrase */
 			if (op[1] != 0) {
 			    pgppass = strdup(op + 1);
@@ -1388,22 +1453,23 @@ main(argc, argv)
                             fprintf(stderr, "\n");
 			}
 			break;
+#endif
 		}
 	    }
 	}
 
 #ifdef HALF_DUPLEX
 	{
-	    struct hostent *h;
-	    char host[512];
-
-	    gethostname(host, sizeof host);
-	    h = gethostbyname(host);
-	    bcopy((char *) (h->h_addr), (char *) &localhost,
-		sizeof localhost);
+#ifndef INADDR_LOOPBACK
+	/* The standard local host loopback address is supposed to be
+	   defined in <netinet/in.h>, but just in case some screwball
+           system doesn't, let's include our own definition here. */
+#define INADDR_LOOPBACK  0x7F000001
+#endif
+	localhost.s_addr = htonl(INADDR_LOOPBACK);
 #ifdef HDX_DEBUG
 	    if (Debug) {
-                fprintf(stderr, "%s: local host %s = %s\n", prog, host,
+                fprintf(stderr, "%s: local host %s\n", prog,
 		       inet_ntoa(localhost));
 	    }
 #endif
@@ -1418,9 +1484,7 @@ main(argc, argv)
 					 older libraries. */
 #endif
 
-#ifndef OLD2X
 	rate_start(4000, 8000);
-#endif
 
 	/* Initialise GSM decoding. */
 
@@ -1599,6 +1663,9 @@ main(argc, argv)
 		c->pgpkey[0] = FALSE;
 		bzero(c->keymd5, 16);
 		c->con_uname[0] = c->con_email[0] = 0;
+#ifndef CRYPTO
+		c->con_crypt_warning = FALSE;
+#endif
 		conn = c;
 		bcopy(&from.sin_addr, &(c->con_addr),
 		    sizeof(struct in_addr));
@@ -1652,7 +1719,7 @@ main(argc, argv)
 		    soundterm();
 		    audiok = FALSE;
 		    if (Debug) {
-                         fprintf(stderr, "%s: releasing audio before busy signel fork().\n", prog);
+                         fprintf(stderr, "%s: releasing audio before busy signal fork().\n", prog);
 		    }
 		}
 		cpid = fork();
@@ -1732,15 +1799,17 @@ main(argc, argv)
 	   packets from it until the busy timeout expires. */
 
 	if (c->con_busy > 0) {
-if (Debug) {
-    fprintf(stderr, "%s: busy--discarding packet from %s.\n", prog, 
-	c->con_hostname);
-}
+	    if (Debug) {
+                fprintf(stderr, "%s: busy--discarding packet from %s.\n", prog, 
+			c->con_hostname);
+	    }
 	    continue;
 	}
 
 	wasrtp = FALSE;
 
+#ifdef CRYPTO
+
         /* If a DES key is present and we're talking RTP or VAT
 	   protocol we must decrypt the packet at this point.
 	   We decrypt the packet if:
@@ -1824,6 +1893,7 @@ if (Debug) {
 #endif
 	    }
 	}
+#endif
 
 	/* If this packet arrived on the session control port, dispatch
 	   it to the appropriate handler for its protocol. */
@@ -2134,6 +2204,22 @@ if (Debug) {
 	    }
 	}
 
+	
+	if ((replycmdexe != NULL && !c->con_reply_current)) {
+            char *rcmd = StrReplace(replycmdexe, "%s", c->con_hostname);
+
+	    if (rcmd != NULL) {
+		if (Debug) {
+                    fprintf(stderr, "Executing: %s\n", rcmd);
+		}
+		system(rcmd);
+		c->con_reply_current = TRUE;
+		if (rcmd != replycmdexe) {
+		    free(rcmd);
+		}
+	    }
+	}
+
         /* If this is the first sound packet we've seen from this host 
 	   in this protocol, create (or update) the reply command file. */
 
@@ -2169,8 +2255,8 @@ if (Debug) {
 
 	/* If this is a half duplex mute request, immediately release
 	   the audio output device if we have it.  We verify the
-           sender's address and accept mute requests only from the
-	   local host; nobody else has any business telling us
+           sender's address and accept mute requests only frome
+	   localhost; nobody else has any business telling us
 	   to shut up! */
 
 	if (sb.compression & fHalfDuplex) {
@@ -2180,7 +2266,7 @@ if (Debug) {
                                            "Bogus(3)" };
 
                 fprintf(stderr, "%s: half-duplex %s request from %s.\n",
-		    prog, hdxreq[sb.compression & 3], sb.sendinghost);
+		    prog, hdxreq[sb.compression & 3], inet_ntoa(from.sin_addr));
 	    }
 #endif
 	    if (memcmp(&from.sin_addr, &localhost, sizeof localhost) == 0) {
@@ -2410,6 +2496,7 @@ if (Debug) {
 	}
 
 
+#ifdef CRYPTO
 	if ((sb.compression & fKeyPGP)) {
 	    char cmd[256], f[40], kmd[16];
 	    FILE *kfile;
@@ -2502,13 +2589,17 @@ if (Debug) {
 		    unlink(f);
 		}
 	    }
-	} else {
+	} else
+#endif
+	{
 	    playbuffer(&sb, c);
 	}
     }
 #ifdef MEANS_OF_EXIT
     close(sock);
+#ifdef CRYPTO
     desdone();
+#endif
     gsm_destroy(gsmh);
     lpc_end();
     exiting();
