@@ -8,4 +8,4 @@
     die a horrible and difficult to diagnose death if it isn't so.  */
 
 #define Relno   \
-"Release 7.0, May 1999"
+"Release 7.2, June MM"
